namespace Unibas.Questionari.Controllo {

using Unibas.Questionari.Modello;
using Unibas.Questionari.Persistenza;
    
public class ControlloSecondario {
    
    public void SchermoCompilaQuestionario(Risposte risposte, Questionario questionario) {
        if (questionario == null || risposte == null) {
            throw new System.ArgumentException("Questionario nullo");
        }
        risposte.IdQuestionario = questionario.Id;
        for (int i = 0; i < questionario.NumeroQuesiti; i++) {
            Quesito quesito = questionario.GetQuesito(i);
            StampaQuesito(quesito, i);
            StampaPrecedenteRisposta(risposte, i);
            string risposta = LeggiRisposta(quesito.NumeroRisposte);
            if (risposta.Equals("stop")) {
                return;
            }
            AggiornaRisposta(risposta, i, risposte);
        }
    }

    private void StampaQuesito(Quesito quesito, int i) {
        System.Console.WriteLine("------------------------------------");
        System.Console.WriteLine("         Quesito n. " + i);
        System.Console.WriteLine("------------------------------------");
        System.Console.WriteLine(quesito);
    }
        
    private void StampaPrecedenteRisposta(Risposte risposte, int i) {
        if (risposte.NumeroRisposte > i) {
            string risposta = risposte[i];
            if (!risposta.Equals("")) {
                System.Console.WriteLine("Risposta precedentemente fornita: " + risposta);
            }
        }
    }

    private string LeggiRisposta(int numeroRisposte) {
        System.Console.Write("Risposta ? (\"stop\" per uscire) ----> ");
        string scelta = Unibas.Utilita.Console.LeggiStringa();
        while (!scelta.Equals("") && !scelta.Equals("stop") && !VerificaLettera(scelta, numeroRisposte)) {
            System.Console.Write("   Errore. Ripeti ----> ");
            scelta = Unibas.Utilita.Console.LeggiStringa();
        }
        return scelta;
        
    }
    
    private bool VerificaLettera(string scelta, int numeroRisposte) {
        if (scelta.Length != 1) {
            return false;
        }
			int valore = (int)(scelta[0] - 97);
        return ((valore >= 0) && (valore < numeroRisposte));
    }
        
    private void AggiornaRisposta(string risposta, int i, Risposte risposte) {
        if (risposte.NumeroRisposte > i) {
            risposte[i] = risposta;
        } else {
            risposte.AddRisposta(risposta);
        }
    }

   public void SchermoValutaQuestionario(Risposte risposte, Questionario questionario) {
        if (risposte == null || questionario == null) {
            throw new System.ArgumentException("E' necessario compilare il questionario prima di valutarlo");
        }
        Valutatore valutatore = new Valutatore(questionario, risposte);
        System.Console.WriteLine("\n" + risposte + "\n");
        System.Console.WriteLine("Risposte esatte:    " + valutatore.NumeroRisposteEsatte);
        System.Console.WriteLine("Risposte non date:  " + valutatore.NumeroRisposteNonDate);
        System.Console.WriteLine("Risposte errate:    " + valutatore.NumeroRisposteErrate);
    }    
                
}

}
