namespace Unibas.Questionari.Modello {

public class Valutatore {

    private Questionario questionario;
    private Risposte risposte;
    
    public Valutatore(Questionario questionario, Risposte risposte) {
        this.questionario = questionario;
        this.risposte = risposte;
    }

    public int NumeroRisposte {
        get {
            return this.risposte.NumeroRisposte;
        }
    }

    public string GetRisposta(int i) {
        System.Diagnostics.Debug.Assert(i >= 0 && i < this.NumeroRisposte, "Posizione inesistente: " + i);
        return this.risposte[i];
    }
    
    public string GetSoluzione(int i) {
        System.Diagnostics.Debug.Assert(i >= 0 && i < this.questionario.NumeroQuesiti, "Posizione inesistente: " + i);
        Quesito quesitoIesimo = this.questionario.GetQuesito(i);
        return quesitoIesimo.Soluzione;
    }

    public string GetDifficolta(int i) {
        System.Diagnostics.Debug.Assert(i >= 0 && i < this.NumeroRisposte, "Posizione inesistente: " + i);
        Quesito quesitoIesimo = this.questionario.GetQuesito(i);
        return quesitoIesimo.IdDifficolta;
    }

    public bool IsEsatta(int i) {
        System.Diagnostics.Debug.Assert(i >= 0 && i < this.NumeroRisposte, "Posizione inesistente: " + i);
        return (this.GetRisposta(i).Equals(this.GetSoluzione(i)));
    }
    
    public bool IsNonData(int i) {
        System.Diagnostics.Debug.Assert(i >= 0 && i < this.NumeroRisposte, "Posizione inesistente: " + i);
        return (this.GetRisposta(i).Equals(""));
    }
        
    public int NumeroRisposteEsatte {
        get {
            int risposteEsatte = 0;
            for (int i = 0; i < this.NumeroRisposte; i++) {
                if (this.GetRisposta(i).Equals(this.GetSoluzione(i))) {
                    risposteEsatte++;
                }
            }
            return risposteEsatte;
        }
    }

    public int NumeroRisposteErrate {
        get {
            int risposteErrate = 0;
            for (int i = 0; i < this.NumeroRisposte; i++) {
                if (!this.GetRisposta(i).Equals("") &&
                    !this.GetRisposta(i).Equals(this.GetSoluzione(i))) {
                    risposteErrate++;
                }
            }
            return risposteErrate;
        }
    }
    
    public int NumeroRisposteNonDate {
        get {
            int quesiti = questionario.NumeroQuesiti;
            int risposteDate = this.NumeroRisposteEsatte + this.NumeroRisposteErrate;
            return quesiti - risposteDate;
        }
    }
    
}

}
