namespace Unibas.Questionari.Persistenza {

using System.Xml;	
using Unibas.Questionari.Modello;
	
public class DAOQuestionario {
    
    public static Questionario CaricaQuestionario(string nomeFile) {
        System.Xml.XmlDocument document = CostruisciDOM(nomeFile);
        Questionario questionario = new Questionario();
        AnalizzaRadice(document, questionario);
        RiempiArgomenti(document, questionario);
        RiempiDifficolta(document, questionario);
        RiempiQuesiti(document, questionario);
        return questionario;
    }

    private static XmlDocument CostruisciDOM(string nomeFile) {
		XmlDocument document = new XmlDocument();
        XmlTextReader textReader = null;
        XmlValidatingReader reader = null;
        try {
        	textReader = new XmlTextReader(nomeFile);
        	reader = new XmlValidatingReader(textReader);
        	document.Load(reader);
        } catch (XmlException xe) {
            throw new DAOException(xe.ToString());
        } catch (System.Xml.Schema.XmlSchemaException se) {
            throw new DAOException(se.ToString());            
        } catch (System.IO.IOException ioe) {
            throw new DAOException(ioe.ToString());            
        } finally {
        	if (reader != null) {
        		reader.Close();
        	}
		}
        return document;
    }
    
    private static void AnalizzaRadice(XmlDocument document, Questionario questionario) {
        XmlElement radice = document.DocumentElement;
        string id = radice.GetAttribute("id");
        string nomeDisciplina = radice.GetAttribute("disciplina");
        questionario.Id = id;
        questionario.Disciplina = nomeDisciplina;
    }
    
    private static void RiempiArgomenti(XmlDocument document, Questionario questionario) {
        XmlElement radice = document.DocumentElement;
        XmlElement listaArgomenti = GetFirstChildByName(radice, "listaArgomenti");
        XmlNodeList argomenti = listaArgomenti.GetElementsByTagName("argomento");
        for(int i = 0; i < argomenti.Count; i++){
            XmlElement elementoArgomento = (XmlElement)argomenti[i];
            Argomento argomento = RiempiArgomento(elementoArgomento);
            questionario.AddArgomento(argomento);
        }
    }
    
    private static Argomento RiempiArgomento(XmlElement elementoArgomento) {
        string id = elementoArgomento.GetAttribute("id");
        string nome = elementoArgomento.GetAttribute("nome");
        Argomento argomento = new Argomento();
        argomento.Id = id;
        argomento.Nome = nome;
        return argomento;
    }
    
    private static void RiempiDifficolta(XmlDocument document, Questionario questionario) {
        XmlElement radice = document.DocumentElement;
        XmlElement listaDifficolta = GetFirstChildByName(radice, "listaDifficolta");
        XmlNodeList nldifficolta = listaDifficolta.GetElementsByTagName("difficolta");
        for(int i = 0; i < nldifficolta.Count; i++){
            XmlElement elementoDifficolta = (XmlElement)nldifficolta[i];
            Difficolta difficolta = RiempiDifficolta(elementoDifficolta);
            questionario.AddDifficolta(difficolta);
        }
    }
    
    private static Difficolta RiempiDifficolta(XmlElement elementoDifficolta) {
        string id = elementoDifficolta.GetAttribute("id");
        string descrizione = elementoDifficolta.GetAttribute("descrizione");
        Difficolta difficolta = new Difficolta();
        difficolta.Id = id;
        difficolta.Descrizione = descrizione;
        return difficolta;
    }
    
    private static void RiempiQuesiti(XmlDocument document, Questionario questionario) {
        XmlElement radice = document.DocumentElement;
        XmlElement elencoQuesiti = GetFirstChildByName(radice, "listaQuesiti");
        XmlNodeList quesiti = elencoQuesiti.GetElementsByTagName("quesito");
        for(int i = 0; i < quesiti.Count; i++){
            XmlElement elementoQuesito = (XmlElement)quesiti[i];
            Quesito quesito = RiempiQuesito(elementoQuesito);
            questionario.AddQuesito(quesito);
        }
    }
    
    private static Quesito RiempiQuesito(XmlElement elementoQuesito) {
        string id = elementoQuesito.GetAttribute("id");
        string idArgomento = elementoQuesito.GetAttribute("argomento");
        string idDifficolta = elementoQuesito.GetAttribute("difficolta");
        Quesito quesito = new Quesito();
        quesito.Id = id;
        quesito.IdArgomento = idArgomento;
        quesito.IdDifficolta = idDifficolta;
        RiempiDomanda(elementoQuesito, quesito);
        RiempiRisposte(elementoQuesito, quesito);
        return quesito;
    }
    
    private static void RiempiDomanda(XmlElement elementoQuesito, Quesito quesito) {
    	XmlElement elementoDomanda = GetFirstChildByName(elementoQuesito, "domanda");
    	XmlText testoDomanda = (XmlText)elementoDomanda.FirstChild;
        string valore = testoDomanda.Value.Trim();        
        quesito.Domanda = valore;
    }
    
    private static void RiempiRisposte(XmlElement elementoQuesito, Quesito quesito) {
    	XmlElement elencoRisposte = GetFirstChildByName(elementoQuesito, "listaRisposte");
        string soluzione = elencoRisposte.GetAttribute("soluzione");
        quesito.Soluzione = soluzione;
        XmlNodeList risposte = elencoRisposte.GetElementsByTagName("risposta");
        for(int i = 0; i < risposte.Count; i++){
            XmlElement elementoRisposta = (XmlElement)risposte[i];
        	XmlText testoRisposta = (XmlText)elementoRisposta.FirstChild;
            string risposta = testoRisposta.Value.Trim();
            quesito.AddRisposta(risposta);
        }
    }
    
    private static XmlElement GetFirstChildByName(XmlElement elemento, string nome) {
    	XmlNodeList figli = elemento.ChildNodes;
        for (int i = 0; i < figli.Count; i++) {
    	    XmlNode figlio = figli[i];
            if (figlio.Name.Equals(nome)) {
                return (XmlElement)figlio;
            }
        }
        return null;
    }

}

}
