namespace Unibas.Questionari.Persistenza {

using System.Xml;
using Unibas.Questionari.Modello;
   
public class DAORisposte {
    
    public static void SalvaRisposte(Risposte risposte, string nomeFile)  {
        if (risposte == null) {
            throw new DAOException("Le risposte non possono essere nulle");
        }
        XmlDocument document = CreaDOM();
        CompletaRadice(risposte, document);
        AggiungiListaRisposte(risposte, document);
        SalvaDocumento(document, nomeFile);
        string percorso = DAOUtilita.EstraiPercorso(nomeFile);
        DAOUtilita.CopiaDTD(percorso);
    }
        
    private static XmlDocument CreaDOM() {
        XmlDocument document = new XmlDocument();
        XmlElement radice = (XmlElement)document.CreateElement("risposte");
        document.AppendChild(radice);
        return document;
    }
    
    private static void CompletaRadice(Risposte risposte, XmlDocument document) {
        XmlElement radice = document.DocumentElement;
        radice.SetAttribute("idQuestionario", risposte.IdQuestionario);
    }

    private static void AggiungiListaRisposte(Risposte risposte, XmlDocument document) {
		XmlElement radice = (XmlElement)document.FirstChild;   
        XmlElement elementoListaRisposte = document.CreateElement("listaRisposte");
        radice.AppendChild(elementoListaRisposte);
        for (int i = 0; i < risposte.NumeroRisposte; i++) {
            AggiungiRisposta(risposte, i, elementoListaRisposte, document);
        }
    }
    
    private static void AggiungiRisposta(Risposte risposte, int i, 
    		                             XmlElement elementoListaRisposte,
										 XmlDocument document) {
    	XmlElement elementoRisposta = document.CreateElement("risposta");
    	string risposta = risposte[i];
        XmlAttribute attrId = document.CreateAttribute("id");
        attrId.Value = "r" + i;
        elementoRisposta.SetAttributeNode(attrId);
        XmlText testo = document.CreateTextNode(risposta);
        elementoRisposta.AppendChild(testo);
    	elementoListaRisposte.AppendChild(elementoRisposta);
    }
    
    
    private static void SalvaDocumento(XmlDocument document, string nomeFile) {
        XmlTextWriter writer = new XmlTextWriter(nomeFile, null);
        writer.Formatting = Formatting.Indented;
        try {
            document.Save(writer);
        } catch (XmlException e) {
            throw new DAOException(e.ToString());
        } finally {
            if (writer != null) {
                writer.Close();
            }
        }
    }

}
	
}
