namespace Unibas.Questionari.Test {

using NUnit.Framework;	
using Unibas.Questionari.Modello;
using Unibas.Questionari.Persistenza;
	
[TestFixture]
public class TestDAOQuestionario {

    private string nomeFile1 = Utilita.nomeFile1;
    private string nomeFile2 = Utilita.nomeFile2;
    private string nomeFile3 = Utilita.nomeFile3;
    private string nomeFile4 = Utilita.nomeFile4;
    private Questionario questionario;
    
    [SetUp]
    protected void setUp() {
    }
    
    [Test]
    public void TestCaricaQuestionario1() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile1);
        } catch (System.Exception e) {
            System.Console.WriteLine(e);
            Assert.Fail();
        }
        VerificaQuestionario(questionario, "test1", "Disciplina di prova", 1, 3, 1);
    }
            
    [Test]
    public void TestCaricaQuestionario2() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile2);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        VerificaQuestionario(questionario, "geo", "Geografia", 3, 3, 4);
    }

    [Test]
    public void TestCaricaQuestionario3() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile3);
            Assert.Fail();
        } catch (DAOException daoe) {
            Assert.IsTrue(daoe != null);
            //System.Console.WriteLine(daoe);
        }
    }

    [Test]
    public void TestCaricaQuestionario5() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        VerificaQuestionario(questionario, "geo", "Geografia", 3, 3, 9);
    }

    
    [Test]
    public void TestCaricaArgomento1() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile1);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        Argomento argomento = questionario.GetArgomento("yyy");
        VerificaArgomento(questionario, argomento, "yyy", "YYY");
    }

    [Test]
    public void TestCaricaArgomento2() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        Argomento argomento = questionario.GetArgomento("naz");
        VerificaArgomento(questionario, argomento, "naz", "Nazioni Europee");
    }
    
    [Test]
    public void TestCaricaDifficolta1() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile1);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        Difficolta difficolta = questionario.GetDifficolta("d1");
        VerificaDifficolta(questionario, difficolta, "d1", "Domanda di difficolta' media");
        difficolta = questionario.GetDifficolta("d2");
        VerificaDifficolta(questionario, difficolta, "d2", "Domanda di difficolta' avanzata");
    }

    [Test]
    public void TestCaricaDifficolta2() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        Difficolta difficolta = questionario.GetDifficolta("d1");
        VerificaDifficolta(questionario, difficolta, "d1", "Domanda di difficolta' media");
        difficolta = questionario.GetDifficolta("d2");
        VerificaDifficolta(questionario, difficolta, "d2", "Domanda di difficolta' avanzata");
    }

    [Test]
    public void TestCaricaQuesito1() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile1);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        Quesito quesito = questionario.GetQuesito(0);
        VerificaQuesito(quesito, "xxx", "Questo e' il testo della domanda ?", 4,
                        "aaa", "bbb", "ccc", "b", "yyy", "d1");
        Assert.IsTrue(quesito.GetRisposta(3).Equals("ddd"), "risposta d");
        try {
            quesito.GetRisposta(4);
            Assert.Fail();
        } catch (System.Exception e) {
            Assert.IsTrue(e != null, "risposta e");
        }
    }

    [Test]
    public void TestCaricaQuesito2() {
        questionario = null;
        try {
            questionario = DAOQuestionario.CaricaQuestionario(nomeFile4);
        } catch (DAOException daoe) {
            System.Console.WriteLine(daoe);
            Assert.Fail();
        }
        Quesito quesito = questionario.GetQuesito(2);
        VerificaQuesito(quesito, "cap03", "Qual e' la capitale del Regno Unito ?", 3,
                        "Il Cairo", "Roma", "Londra - esatta", "c", "cap", "d2");
    }    

    private void VerificaQuestionario(Questionario questionario, 
                                        string id, string disciplina,
                                        int numeroArgomenti, int numeroDifficolta, 
                                        int numeroQuesiti) {
        Assert.IsTrue(questionario.Id.Equals(id), "id");
        Assert.IsTrue(questionario.Disciplina.Equals(disciplina), "disciplina");
        Assert.IsTrue(questionario.NumeroArgomenti == numeroArgomenti, "numero argomenti");
        Assert.IsTrue(questionario.NumeroDifficolta == numeroDifficolta, "numero difficolta");
        Assert.IsTrue(questionario.NumeroQuesiti == numeroQuesiti, "numero quesiti");
    }
        
    private void VerificaArgomento(Questionario questionario,
                                    Argomento argomento, string id, 
                                    string nome) {
        Assert.IsTrue(argomento.Id.Equals(id), "id");
        Assert.IsTrue(argomento.Nome.Equals(nome), "nome argomento");                                         
    }

    private void VerificaDifficolta(Questionario questionario,
                                    Difficolta difficolta, string id,
                                    string descrizione) {
        Assert.IsTrue(difficolta.Id.Equals(id), "id");
        Assert.IsTrue(difficolta.Descrizione.Equals(descrizione), "descrizione"); 
    }
    
    private void VerificaQuesito(Quesito quesito, string id, 
                                    string domanda, int numeroRisposte,
                                    string rispostaA, string rispostaB,
                                    string rispostaC, string soluzione,
                                    string idArgomento, string idDifficolta) {
        Assert.IsTrue(quesito.Id.Equals(id), "id quesito"); 
        Assert.IsTrue(quesito.Domanda.Equals(domanda), "domanda: ");
        Assert.IsTrue(quesito.NumeroRisposte == numeroRisposte, "numero risposte");
        Assert.IsTrue(quesito.GetRisposta(0).Equals(rispostaA), "risposta a");
        Assert.IsTrue(quesito.GetRisposta(1).Equals(rispostaB), "risposta b");
        Assert.IsTrue(quesito.GetRisposta(2).Equals(rispostaC), "risposta c");
        Assert.IsTrue(quesito.Soluzione.Equals(soluzione), "risposta esatta");
        string idArgomentoQ = quesito.IdArgomento;
        Assert.IsTrue(idArgomentoQ.Equals(idArgomento), "argomento quesito");        
        string idDifficoltaQ = quesito.IdDifficolta;
        Assert.IsTrue(idDifficoltaQ.Equals(idDifficolta), "difficolta");        
    }
    
}

}
